package dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.special;

import java.util.function.Function;
import com.mojang.datafixers.kinds.Applicative;
import com.mojang.datafixers.kinds.App;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import com.google.common.base.Preconditions;
import net.minecraft.ChatFormatting;
import net.minecraft.util.StringUtil;
import net.minecraft.network.chat.Component;
import dev.shadowsoffire.apotheosis.Apoth;
import dev.shadowsoffire.apotheosis.adventure.affix.Affix;
import net.minecraft.world.entity.projectile.AbstractArrow;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.item.ItemStack;
import java.util.Set;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemClass;
import com.google.common.collect.ImmutableSet;
import dev.shadowsoffire.apotheosis.adventure.loot.LootCategory;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import java.util.Map;
import com.mojang.serialization.Codec;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.GemBonus;

public class BloodyArrowBonus extends GemBonus
{
    public static Codec<BloodyArrowBonus> CODEC;
    protected final Map<LootRarity, Data> values;
    
    public BloodyArrowBonus(final Map<LootRarity, Data> values) {
        super(Apotheosis.loc("bloody_arrow"), new GemClass("ranged_weapon", (Set)ImmutableSet.of(LootCategory.BOW, LootCategory.CROSSBOW)));
        this.values = values;
    }
    
    public void onArrowFired(final ItemStack gem, final LootRarity rarity, final LivingEntity user, final AbstractArrow arrow) {
        final Data d = this.values.get(rarity);
        if (Affix.isOnCooldown(this.getCooldownId(gem), d.cooldown, user)) {
            return;
        }
        user.m_6469_(user.m_269291_().m_269079_(Apoth.DamageTypes.CORRUPTED), user.m_21233_() * d.healthCost);
        arrow.m_36781_(arrow.m_36789_() * d.dmgMultiplier);
        Affix.startCooldown(this.getCooldownId(gem), user);
    }
    
    public Codec<? extends GemBonus> getCodec() {
        return BloodyArrowBonus.CODEC;
    }
    
    public Component getSocketBonusTooltip(final ItemStack gem, final LootRarity rarity) {
        final Data d = this.values.get(rarity);
        final Component cooldown = (Component)Component.m_237110_("affix.apotheosis.cooldown", new Object[] { StringUtil.m_14404_(d.cooldown) });
        return (Component)Component.m_237110_("bonus." + this.getId() + ".desc", new Object[] { Affix.fmt(d.healthCost * 100.0f), Affix.fmt(100.0f * d.dmgMultiplier), cooldown }).m_130940_(ChatFormatting.YELLOW);
    }
    
    public BloodyArrowBonus validate() {
        Preconditions.checkNotNull(this.values);
        this.values.forEach((k, v) -> {
            Preconditions.checkNotNull(k);
            Preconditions.checkNotNull(v);
            return;
        });
        return this;
    }
    
    public boolean supports(final LootRarity rarity) {
        return this.values.containsKey(rarity);
    }
    
    public int getNumberOfUUIDs() {
        return 0;
    }
    
    static {
        BloodyArrowBonus.CODEC = (Codec<BloodyArrowBonus>)RecordCodecBuilder.create(inst -> inst.group((App)LootRarity.mapCodec((Codec)Data.CODEC).fieldOf("values").forGetter(a -> a.values)).apply((Applicative)inst, (Function)BloodyArrowBonus::new));
    }
    
    record Data(float healthCost, float dmgMultiplier, int cooldown) {
        public static final Codec<Data> CODEC;
        
        static {
            CODEC = RecordCodecBuilder.create(inst -> inst.group((App)Codec.FLOAT.fieldOf("health_cost").forGetter((Function)Data::healthCost), (App)Codec.FLOAT.fieldOf("damage_mult").forGetter((Function)Data::dmgMultiplier), (App)Codec.INT.fieldOf("cooldown").forGetter((Function)Data::cooldown)).apply((Applicative)inst, Data::new));
        }
    }
}
